from gi.repository import Gtk, Gio, Gdk
from widgets.dialogs import OpenFileDialog, SaveFileDialog
from document import Document
from dash import Dash
import os

class Window(Gtk.ApplicationWindow):  
  
    def __init__(self, app):
        Gtk.ApplicationWindow.__init__(self, title="Tahrir",
                                      application=app,
                                      hide_titlebar_when_maximized=True)
        self.set_default_size(960, 560)
        self.set_position(Gtk.WindowPosition.CENTER)
        self.set_icon_from_file('data/tahrir.svg')
        self.fullscreen_status = False
        
        css = Gtk.CssProvider()
        css.load_from_path(os.getcwd()+'/tahrir/gtk-style.css')
        
        context = Gtk.StyleContext()
        context.add_provider_for_screen(Gdk.Screen.get_default(),
                                        css,
                                        Gtk.STYLE_PROVIDER_PRIORITY_USER)
        
        self.box = box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
        self.toolbar = toolbar = Toolbar()
        self.dash = dash = Dash()
        self.notebook = notebook = Gtk.Notebook()
        self.bottombar = bottombar = Bottombar()
        
        self.box.pack_start(toolbar, False, False, 0)
        self.box.pack_start(notebook, True, True, 0)
        self.box.pack_start(bottombar, False, False, 0)
        self.add(box)
        
        notebook.set_show_border(False)
        notebook.set_show_tabs(False)
        notebook.append_page(dash, Gtk.Label('Dash'))
        notebook.show_all()
        notebook.set_current_page(-1)
        self.dash.fill_model(self.get_all_documents())
        
        self.box.show_all()
        bottombar.hide()
        toolbar.set_dash_toolbar()
        
    def add_document(self, filename=None):
        doc = Document(filename)
        self.notebook.append_page(doc, Gtk.Label('Document'))
        self.dash.fill_model(self.get_all_documents())
        self.notebook.show_all()
        self.notebook.set_current_page(-1)
    
    def open_dialog(self):
        d = OpenFileDialog()
        return d
        
    def save_dialog(self):
        d = SaveFileDialog(self)
        return d
                
    def get_current_document(self):
        page = self.notebook.get_current_page()
        child = self.notebook.get_nth_page(page)
        return child
        
    def get_all_documents(self):
        pages = self.notebook.get_n_pages()
        docs = []
        page = 0
        while page < pages:
            docs.append(self.notebook.get_nth_page(page))
            page += 1
        docs.remove(self.dash)
        return docs


class Toolbar(Gtk.Toolbar):
  
    def __init__(self):
        Gtk.Toolbar.__init__(self)
        self.get_style_context().add_class('primary-toolbar')
        self.set_icon_size(Gtk.IconSize.BUTTON)
        self.set_size_request(-1, 42)
        
        left_box = Gtk.Box()
        left_item = Gtk.ToolItem()
        left_item.add(left_box)
        
        self.new = new = ToolbarButton('accessories-text-editor-symbolic', 'Create new Document')        
        self.open_btn = open_btn = ToolbarButton('document-open-symbolic', 'Open file')        
        self.dash = dash = ToolbarButton('input-dialpad-symbolic', 'Dash')                
        self.save = save = ToolbarButton('document-save-symbolic', 'Save Document')
        self.run = run = ToolbarButton('system-run-symbolic', 'Run current file')
        
        left_box.pack_start(new, False, False, 3)
        left_box.pack_start(self.open_btn, False, False, 3)
        left_box.pack_start(dash, False, False, 3)
        left_box.pack_start(save, False, False, 3)        
        left_box.pack_start(run, False, False, 3)        
        
        spacer = Gtk.ToolItem()
        spacer.set_expand(True)
        
        search = ToolbarSearchBox()
        
        spacer2 = Gtk.ToolItem()
        spacer2.set_expand(True)
        
        right_box = Gtk.Box()
        right_item = Gtk.ToolItem()
        right_item.add(right_box)
        
        self.lines = lines = ToolbarButton('view-list-symbolic', 'Show/hide line numbers')        
        self.lock = lock = ToolbarButton('changes-allow-symbolic', 'Lock Document')
        self.settings = settings = ToolbarButton('preferences-other-symbolic', 'Settings')
        fullscreen = ToolbarButton('view-fullscreen-symbolic', 'Fullscreen Mode')
        
        right_box.pack_start(lines, False, False, 3)
        right_box.pack_start(lock, False, False, 3)
        right_box.pack_start(settings, False, False, 3)
        right_box.pack_start(fullscreen, False, False, 3)        
        

        self.insert(left_item, -1)
        self.insert(spacer, -1)
        self.insert(search, -1)
        self.insert(spacer2, -1)        
        self.insert(right_item, -1)
  
        new.connect('clicked', self._on_new_clicked)
        open_btn.connect('clicked', self._on_open_clicked)
        dash.connect('clicked', self._on_dash_clicked)
        save.connect('clicked', self._on_save_clicked)
        run.connect('clicked', self._on_run_clicked)
        lines.connect('clicked', self._on_line_numbers_clicked)
        lock.connect('clicked', self._on_locker_clicked)
        fullscreen.connect('clicked', self._on_fullscreen_clicked)
        
        self.set_dash_toolbar()
        
    def set_dash_toolbar(self):
        self.save.set_visible(False)
        self.dash.set_visible(False)
        self.run.set_visible(False)
        self.lines.set_visible(False)
        self.lock.set_visible(False)
        self.new.set_visible(True)
        self.open_btn.set_visible(True)
        
    def set_document_toolbar(self):
        self.save.set_visible(True)
        self.dash.set_visible(True)
        self.run.set_visible(True)
        self.lines.set_visible(True)
        self.lock.set_visible(True)
        self.new.set_visible(False)
        self.open_btn.set_visible(False)
        
    def set_save_visible(self, val):
        self.save.set_visible(val)
        
    def _on_new_clicked(self, btn):
        window = self.get_parent().get_parent()
        window.add_document()
        self.set_document_toolbar()
        
    def _on_open_clicked(self, btn):
        window = self.get_parent().get_parent()
        d = window.open_dialog()
        if d.run() == 1:
            filename = d.get_filename()
            window.add_document(filename)
            window.dash.fill_model(window.get_all_documents())
            self.set_document_toolbar()
        d.destroy()
        
    def _on_dash_clicked(self, btn):
        window = self.get_parent().get_parent()
        window.notebook.set_current_page(0)
        self.set_dash_toolbar()
            
    def _on_save_clicked(self, btn):
        window = self.get_parent().get_parent()
        doc = window.get_current_document()
        r = doc.save_request()
        if r == True:
            docs = window.get_all_documents()
            window.dash.fill_model(docs)
        
    def _on_run_clicked(self, btn):
        window = self.get_parent().get_parent()
        doc = window.get_current_document()
        if doc.save_request() == True:            
            os.system(doc.filename)
        
    def _on_line_numbers_clicked(self, btn):
        window = self.get_parent().get_parent()
        docs = window.get_all_documents()
        for doc in docs:
            doc.inverse_line_numbers()
        
    def _on_locker_clicked(self, btn):
        window = self.get_parent().get_parent()
        docs = window.get_all_documents()
        for doc in docs:
            if doc.get_locked() == True:
                doc.set_locked(False)
                self.update_locker_icon(True)
            else:
                doc.set_locked(True)
                self.update_locker_icon(False)
            
    def _on_fullscreen_clicked(self, btn):
        window = self.get_parent().get_parent()
        if window.fullscreen_status == True:
            window.unfullscreen()
            window.fullscreen_status = False
        else:
            window.fullscreen()
            window.fullscreen_status = True
            
    def update_locker_icon(self, allow):
        if allow == True:
            gicon = Gio.ThemedIcon.new_with_default_fallbacks('changes-allow-symbolic')
        else:
            gicon = Gio.ThemedIcon.new_with_default_fallbacks('changes-prevent-symbolic')
        img = Gtk.Image.new_from_gicon(gicon,
                                      Gtk.IconSize.MENU)
        self.lock.set_image(img)        
  
        
class ToolbarSearchBox(Gtk.ToolItem):
  
    def __init__(self):
        Gtk.ToolItem.__init__(self)
        entry = Gtk.Entry()
        entry.set_size_request(300, -1)
        entry.set_placeholder_text('Type to search')
        gicon = Gio.ThemedIcon.new_with_default_fallbacks('edit-find-symbolic')
        entry.set_icon_from_gicon(Gtk.EntryIconPosition.SECONDARY,
                                  gicon)
        self.add(entry)
        

class ToolbarButton(Gtk.Button):  
  
    def __init__(self, icon_name, tooltip):
        Gtk.Button.__init__(self)
        self.set_tooltip_text(tooltip)
        self.set_size_request(32, -1)
        self.set_relief(Gtk.ReliefStyle.NONE)
        self.set_can_focus(False)
        gicon = Gio.ThemedIcon.new_with_default_fallbacks(icon_name)
        img = Gtk.Image.new_from_gicon(gicon,
                                      Gtk.IconSize.MENU)
        self.set_image(img)


class Bottombar(Gtk.EventBox):
  
    def __init__(self):
        Gtk.EventBox.__init__(self)
        #self.get_style_context().add_class('white-bg')
        self.get_style_context().add_class('menubar')
        box = Gtk.Box()
        self.add(box)
        
        tab_width = 4
        tab_label = Gtk.Label('Tab Width: ')
        widthes = Gtk.ComboBoxText()
        #widthes.set_entry('Title')
        #widthes.set_entry_text_column('Title')
        model = widthes.get_model()
        widthes.set_model(model)
        
        
        line = 1
        col = 1
        self.label = label = Gtk.Label('Ln %d,  Col %d' % (line, col))
        box.pack_end(Gtk.Label(''), False, False, 9)
        box.pack_end(label, False, False, 3)
        box.pack_end(tab_label, False, False, 3)
        box.pack_end(widthes, False, False, 3)
        
        
    def update_label(self, line, col):
        self.label.set_text('Ln %d,  Col %d' % (line, col))
        self.label.show()
  
                      
class App(Gtk.Application):
  
    def __init__(self):
        Gtk.Application.__init__(self)
        
    def do_activate(self):
        self.win = win = Window(self)
        win.present()
        win.dash.hide()
        win.notebook.grab_focus()

    def do_startup(self):
        Gtk.Application.do_startup(self)
        menu = Gio.Menu()        
        
        menu.append('New Document', 'app.new')
        menu.append('About Tahrir', 'app.about')
        menu.append('Quit', 'app.quit')
        self.set_app_menu(menu)
        
        about_action = Gio.SimpleAction.new('about', None)
        about_action.connect('activate', self._about)
        self.add_action(about_action)
        
        new_action = Gio.SimpleAction.new('new', None)
        new_action.connect('activate', self._new)
        self.add_action(new_action)
        
        quit_action = Gio.SimpleAction.new('quit', None)
        quit_action.connect('activate', self._quit)
        self.add_action(quit_action)
        
    def _new(self, action, data):
        pass
        
    def _quit(self, action, data):
        self.quit()

    def _about(self, action, data):
        about = Gtk.AboutDialog()
        about.set_title('About Tahrir')
        about.set_program_name('Tahrir')
        about.set_version('2.0')
        about.set_copyright('Created by (c) Eslam Mostafa')
        about.set_comments('Tahrir is a simple but smart text editor, It is designed to give the user\
 the most comfortable experience of writing and editing. Tahrir is designed for Gnu/Linux and written\
 in Python/Gtk.\n Tahrir is an arabic word that means "Freedom" but it also means "Editing", So have fun editing freely.')
        about.set_authors(['Eslam Mostafa <cseslam@gmail.com>'])
        about.set_wrap_license("true")
        about.set_license_type (Gtk.License.GPL_2_0)
        about.set_license("Tahrir is free software;"         
            "you can redistribute it and/or modify it under the terms"
            " of the GNU General Public License as published by the"
            " Free Software Foundation; either version 2 of the"
            " License, or (at your option) any later version.\n"
            "  \n"
            "Tahrir is distributed in the hope that it will be"
            " useful, but WITHOUT ANY WARRANTY; without even the"
            " implied warranty of MERCHANTABILITY or FITNESS FOR"
            " A PARTICULAR PURPOSE.  See the GNU General Public"
            " License for more details.\n"
            "  \n" 
            "You should have received a copy of the GNU General" 
            " Public License along with Tahrir.")
        about.set_modal(True)
        about.set_transient_for(self.win)
        about.connect('response', lambda r, d:about.destroy())
        about.show()
